﻿using Syncfusion.Windows.Forms.Gauge;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace LinearGaugeCustomization
{
    public class CustomRenderer : ILinearGaugeRenderer
    {
        #region Varaibles
        /// </summary>
        /// Gets the LinearGauge
        /// </summary>
        private LinearGauge m_LinearGauge;
        /// <summary>
        /// Gets/Sets the Tick Distance.
        /// </summary>
        private float majorTicksDistance;
        /// <summary>
        /// Calculates the Minor Ticks Pixels.
        /// </summary>
        private float m_minorTicksPixels;
        /// <summary>
        /// Initializes the startpoint variable.
        /// </summary>
        private int startpoint;

        private int majorTicksCount;


        #endregion

        #region Properties
        /// <summary>
        ///  Gets the LinearGauge.
        /// </summary>
        internal LinearGauge LinearGauge
        {
            get
            {
                return m_LinearGauge;
            }
        }
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes the new instance for the MetroThemeLinearGaugeRenderer class.
        /// </summary>
        /// <param name="linearGauge">LinearGauge</param>
        public CustomRenderer(LinearGauge linearGauge)
        {
            m_LinearGauge = linearGauge;
            majorTicksDistance = 0;
            m_minorTicksPixels = 0;
            startpoint = 25;
            this.m_LinearGauge.BackgroundGradientStartColor = Color.White;
            this.m_LinearGauge.BackgroundGradientEndColor = Color.White;
            m_LinearGauge.OuterFrameGradientStartColor = Color.FromArgb(17, 180, 205);
            m_LinearGauge.OuterFrameGradientEndColor = Color.FromArgb(17, 180, 205);
            m_LinearGauge.InnerFrameGradientStartColor = Color.FromArgb(180, 180, 180);
            m_LinearGauge.InnerFrameGradientEndColor = Color.FromArgb(194, 194, 194);
            this.m_LinearGauge.NeedleColor = Color.FromArgb(17, 180, 205);
            this.m_LinearGauge.ScaleLabelColor = Color.Gray;
            majorTicksCount = 0;              
        }
        #endregion

        #region ILinearGaugeMembers

        /// <summary>f
        /// Used to draw Background Frames.
        /// </summary>
        /// <param name="Graphics">Graphics</param>
        public void DrawFrame(Graphics Graphics)
        {

            Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            int tmpSoundCornerRadius = Math.Min(Math.Min(25, this.LinearGauge.Width - 2), this.LinearGauge.Height - 2);
            Rectangle rect = new Rectangle(0, 0, this.LinearGauge.Width - 1, this.LinearGauge.Height - 1);
            Rectangle innerrect = new Rectangle(rect.X + 10, rect.Y + 10, this.LinearGauge.Width - 2 * 10, this.LinearGauge.Height - 2 * 10);

            GraphicsPath graphPath = new GraphicsPath();
            graphPath.AddArc(rect.X, rect.Y, tmpSoundCornerRadius, tmpSoundCornerRadius, 180, 90);
            graphPath.AddArc(rect.X + rect.Width - tmpSoundCornerRadius, rect.Y, tmpSoundCornerRadius, tmpSoundCornerRadius, 270, 90);
            graphPath.AddArc(rect.X + rect.Width - tmpSoundCornerRadius, rect.Y + rect.Height - tmpSoundCornerRadius, tmpSoundCornerRadius, tmpSoundCornerRadius, 0, 90);
            graphPath.AddArc(rect.X, rect.Y + rect.Height - tmpSoundCornerRadius, tmpSoundCornerRadius, tmpSoundCornerRadius, 90, 90);
            graphPath.AddLine(rect.X, rect.Y + rect.Height - tmpSoundCornerRadius, rect.X, rect.Y + tmpSoundCornerRadius / 2);

            GraphicsPath graphInnerPath = new GraphicsPath();
            graphInnerPath.AddArc(innerrect.X, innerrect.Y, tmpSoundCornerRadius, tmpSoundCornerRadius, 180, 90);
            graphInnerPath.AddArc(innerrect.X + innerrect.Width - tmpSoundCornerRadius, innerrect.Y, tmpSoundCornerRadius, tmpSoundCornerRadius, 270, 90);
            graphInnerPath.AddArc(innerrect.X + innerrect.Width - tmpSoundCornerRadius, innerrect.Y + innerrect.Height - tmpSoundCornerRadius, tmpSoundCornerRadius, tmpSoundCornerRadius, 0, 90);
            graphInnerPath.AddArc(innerrect.X, innerrect.Y + innerrect.Height - tmpSoundCornerRadius, tmpSoundCornerRadius, tmpSoundCornerRadius, 90, 90);
            graphInnerPath.AddLine(innerrect.X, innerrect.Y + innerrect.Height - tmpSoundCornerRadius, innerrect.X, innerrect.Y + tmpSoundCornerRadius / 2);

            using (LinearGradientBrush brush1 = new LinearGradientBrush(rect,
                    LinearGauge.OuterFrameGradientStartColor,
                   LinearGauge.OuterFrameGradientEndColor,
                    LinearGradientMode.Vertical))
            {
                Graphics.FillPath(brush1, graphPath);
                Graphics.DrawPath(new Pen(LinearGauge.InnerFrameGradientEndColor, 1), graphPath);

            }

            using (LinearGradientBrush brush2 = new LinearGradientBrush(innerrect,
                   LinearGauge.BackgroundGradientStartColor,
                   LinearGauge.BackgroundGradientEndColor,
                   LinearGradientMode.Vertical))
            {
                Graphics.FillPath(brush2, graphInnerPath);
                Graphics.DrawPath(new Pen(LinearGauge.InnerFrameGradientStartColor, 1.6f), graphInnerPath);
            }
        }

        /// <summary>
        ///  Used to draw the Tick lines.
        /// </summary>
        /// <param name="Graphics">Graphics</param>
        public void DrawLines(Graphics Graphics)
        {
            Pen majorTickPen = new Pen(LinearGauge.MajorTickMarkColor, 2);
            Pen minorTickPen = new Pen(LinearGauge.MinorTickMarkColor);
            Brush brush = new SolidBrush(LinearGauge.ForeColor);
            StringFormat sf = new StringFormat();
            Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.None;

            majorTicksDistance = ((LinearGauge.MaximumValue - LinearGauge.MinimumValue) / LinearGauge.MajorDifference);
            majorTicksCount = ((int)(LinearGauge.MaximumValue - LinearGauge.MinimumValue) / (LinearGauge.MajorDifference)) + 1;
            double majortickValue = LinearGauge.MinimumValue;
            float tickPosition = 25f;
            float temp1 = 0;
            float s = (LinearGauge.MaximumValue - LinearGauge.MinimumValue) % LinearGauge.MajorDifference;
            sf.Alignment = StringAlignment.Center;
            sf.LineAlignment = StringAlignment.Center;
            float minortickValue = 0;
            float tickPositionminor = 0;
            GraphicsPath path = new GraphicsPath();
            int minorcount = LinearGauge.MinorTickCount;
            if (LinearGauge.LinearFrameType == LinearFrameType.Vertical)
            {
                m_minorTicksPixels = ((this.LinearGauge.Height - 50) / majorTicksDistance);
                int x = this.LinearGauge.Width / 2;
                temp1 = 0;
                for (int L = 1; L <= majorTicksCount; L++)
                {
                    Graphics.DrawLine(majorTickPen, x, this.LinearGauge.Height - tickPosition, x - LinearGauge.MajorTicksHeight, this.LinearGauge.Height - tickPosition);
                    if (LinearGauge.ShowScaleLabel)
                    {
                        Graphics.DrawString(this.NumberToStringRepresentation((int)Math.Round(majortickValue, 2)), LinearGauge.Font, brush, new PointF(x - LinearGauge.MajorTicksHeight - 25, this.LinearGauge.Height - tickPosition), sf);
                    }
                    if (L == majorTicksCount)
                        minorcount = (LinearGauge.MinorTickCount * (int)Math.Ceiling(s)) / LinearGauge.MajorDifference;
                    if (majortickValue < LinearGauge.MaximumValue)
                    {
                        for (int S = 1; S <= minorcount; S++)
                        {
                            minortickValue = (m_minorTicksPixels / (LinearGauge.MinorTickCount + 1)) * S;
                            tickPositionminor = this.LinearGauge.Height - (minortickValue + temp1 + 25);
                            Graphics.DrawLine(minorTickPen, x, (float)tickPositionminor, x - LinearGauge.MinorTickHeight, (float)tickPositionminor);
                        }
                        temp1 = m_minorTicksPixels * L;
                    }

                    majortickValue += LinearGauge.MajorDifference;
                    tickPosition += m_minorTicksPixels;
                }
                if (this.LinearGauge.MinimumValue > 0)
                    Graphics.FillRectangle(new SolidBrush(LinearGauge.ValueIndicatorColor), this.LinearGauge.Width / 2 + 10, startpoint + (majorTicksDistance * m_minorTicksPixels) - ((((LinearGauge.Value - this.LinearGauge.MinimumValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels), 5, ((((LinearGauge.Value - this.LinearGauge.MinimumValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels) + 2);
                else
                    Graphics.FillRectangle(new SolidBrush(LinearGauge.ValueIndicatorColor), this.LinearGauge.Width / 2 + 10, startpoint + (majorTicksDistance * m_minorTicksPixels) - ((((Math.Abs(this.LinearGauge.MinimumValue) + LinearGauge.Value) / LinearGauge.MajorDifference)) * m_minorTicksPixels), 5, ((((Math.Abs(this.LinearGauge.MinimumValue) + LinearGauge.Value) / LinearGauge.MajorDifference)) * m_minorTicksPixels) + 2);
                brush.Dispose();
                minorTickPen.Dispose();
            }
            else
            {
                m_minorTicksPixels = ((this.LinearGauge.Width - (startpoint * 2)) / majorTicksDistance);

                int y = this.LinearGauge.Height / 2 - LinearGauge.MajorTicksHeight;

                temp1 = 0;
                int height = y + LinearGauge.MajorTicksHeight;
                for (int L = 1; L <= majorTicksCount; L++)
                {
                    Graphics.DrawLine(majorTickPen, tickPosition, (float)y - 0.5F, tickPosition, (float)height - 0.5F);
                    if (LinearGauge.ShowScaleLabel)
                        Graphics.DrawString(Math.Round(majortickValue, 2).ToString(), LinearGauge.Font, brush, new PointF(tickPosition, y - 10), sf);
                    if (L == majorTicksCount)
                        minorcount = (LinearGauge.MinorTickCount * (int)Math.Ceiling(s)) / LinearGauge.MajorDifference;
                    if (majortickValue < LinearGauge.MaximumValue)
                    {
                        for (int S = 1; S <= minorcount; S++)
                        {
                            minortickValue = (m_minorTicksPixels / (LinearGauge.MinorTickCount + 1)) * S;
                            tickPositionminor = minortickValue + temp1 + startpoint;
                            Graphics.DrawLine(minorTickPen, (float)tickPositionminor, height - 1, (float)tickPositionminor, height - LinearGauge.MinorTickHeight - 1);
                        }
                        temp1 = m_minorTicksPixels * L;
                    }

                    majortickValue += LinearGauge.MajorDifference;
                    tickPosition += m_minorTicksPixels;
                }
                if (this.LinearGauge.MinimumValue > 0)
                    Graphics.FillRectangle(new SolidBrush(LinearGauge.ValueIndicatorColor), 24, this.LinearGauge.Height / 2 + 10, ((((this.LinearGauge.Value - this.LinearGauge.MinimumValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels) + 2, 5);
                else
                    Graphics.FillRectangle(new SolidBrush(LinearGauge.ValueIndicatorColor), 24, this.LinearGauge.Height / 2 + 10, ((((Math.Abs(this.LinearGauge.MinimumValue) + LinearGauge.Value) / LinearGauge.MajorDifference)) * m_minorTicksPixels) + 2, 5);

                Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;


                brush.Dispose();
                minorTickPen.Dispose();
            }

        }

        /// <summary>
        /// Used to draw the range lines.
        /// </summary>
        /// <param name="Graphics">Graphics</param>
        public void DrawRanges(Graphics Graphics)
        {

            Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            if (LinearGauge.LinearFrameType == LinearFrameType.Vertical)
                Graphics.FillRectangle(new SolidBrush(LinearGauge.GaugeBaseColor), this.LinearGauge.Width / 2, startpoint - 1, 5, (((this.majorTicksDistance)) * m_minorTicksPixels) + 2);
            else
                Graphics.FillRectangle(new SolidBrush(LinearGauge.GaugeBaseColor), 24, this.LinearGauge.Height / 2, (((this.majorTicksDistance)) * m_minorTicksPixels) + 2, 5);
            foreach (LinearRange ptrRange in this.LinearGauge.Ranges)
            {
                int rvalve = (int)Math.Ceiling(LinearGauge.MaximumValue - ptrRange.EndValue) / LinearGauge.MajorDifference;
                if (ptrRange.EndValue > ptrRange.StartValue && ptrRange.EndValue <= this.LinearGauge.MaximumValue)
                {
                    if (this.LinearGauge.MinimumValue >= 0 && ptrRange.StartValue < 0)
                    {
                        return;
                    }
                    float startValue = (float)ptrRange.StartValue;
                    float endValue = (float)ptrRange.EndValue;
                    int startwidth = ptrRange.StartWidth;
                    int endwidth = ptrRange.EndWidth;
                    if (this.LinearGauge.MinimumValue < 0)
                    {
                        startValue = this.LinearGauge.MinimumValue + Math.Abs(ptrRange.StartValue);
                    }
                    if (this.LinearGauge.MinimumValue < 0 && ptrRange.StartValue > 0)
                    {
                        startValue = Math.Abs(this.LinearGauge.MinimumValue) + Math.Abs(ptrRange.StartValue);
                    }
                    if (this.LinearGauge.MinimumValue < 0 && ptrRange.StartValue == 0)
                    {
                        startValue = Math.Abs(this.LinearGauge.MinimumValue) + Math.Abs(ptrRange.StartValue);
                        startValue = (((startValue / LinearGauge.MajorDifference)) * m_minorTicksPixels);
                    }
                    if (LinearGauge.LinearFrameType == LinearFrameType.Horizontal)
                    {
                        RectangleF rect = RectangleF.Empty;
                        if (ptrRange.StartValue == 0)
                        {
                            rect = new RectangleF(startpoint + Math.Abs(startValue), this.LinearGauge.Height / 2 + 5, (((ptrRange.EndValue / LinearGauge.MajorDifference)) * m_minorTicksPixels), 5);
                        }
                        else if (ptrRange.StartValue > 0)
                        {
                            rect = new RectangleF(25 + (((startValue / LinearGauge.MajorDifference)) * m_minorTicksPixels), this.LinearGauge.Height / 2 + 5, ((((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels), 5);

                        }
                        else if (ptrRange.StartValue < 0)
                        {
                            startValue = this.LinearGauge.MinimumValue - ptrRange.StartValue;
                            rect = new RectangleF(25 + (((Math.Abs(startValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels), this.LinearGauge.Height / 2, ((((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference)) * m_minorTicksPixels), 5);
                        }
                        GraphicsPath path = new GraphicsPath();
                        PointF pt1, pt2, pt3, pt4;
                        pt1 = new PointF(rect.X, rect.Y);
                        pt2 = new PointF(rect.X, rect.Y - startwidth);
                        pt4 = new PointF(rect.Right, rect.Y);
                        pt3 = new PointF(rect.Right, rect.Y - endwidth);
                        path.AddPolygon(new PointF[] { pt1, pt2, pt3, pt4 });
                        path.Reverse();
                        Graphics.FillPath(new SolidBrush(ptrRange.Color), path);
                    }
                    else
                    {
                        float rangeheight = (ptrRange.EndValue / LinearGauge.MajorDifference) * m_minorTicksPixels;
                        float endvalueRangeHeight = 0f;
                        if (this.LinearGauge.MinimumValue < 0)
                        {
                            rangeheight = ((Math.Abs(this.LinearGauge.MinimumValue) + ptrRange.EndValue) / LinearGauge.MajorDifference) * m_minorTicksPixels;
                        }
                        endvalueRangeHeight = rangeheight;
                        if (this.LinearGauge.MinimumValue < 0 && ptrRange.StartValue == 0)
                        {
                            endvalueRangeHeight = (((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference) * m_minorTicksPixels);
                        }
                        RectangleF rect = RectangleF.Empty;
                        if (ptrRange.StartValue == 0)
                            rect = new RectangleF(this.LinearGauge.Width / 2 + 5, startpoint + (majorTicksDistance * m_minorTicksPixels) - rangeheight, 5, endvalueRangeHeight);
                        else if (ptrRange.StartValue > 0)
                            rect = new RectangleF(this.LinearGauge.Width / 2 + 5, startpoint + (majorTicksDistance * m_minorTicksPixels) - rangeheight, 5, (((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference) * m_minorTicksPixels));
                        else if (ptrRange.StartValue < 0)
                            rect = new RectangleF(this.LinearGauge.Width / 2 + 5, startpoint + (majorTicksDistance * m_minorTicksPixels) - rangeheight, 5, (((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference) * m_minorTicksPixels));
                        GraphicsPath path = new GraphicsPath();
                        PointF pt1, pt2, pt3, pt4;
                        pt1 = new PointF(rect.X, rect.Y);
                        pt2 = new PointF(rect.X - endwidth, rect.Y);
                        pt4 = new PointF(rect.X, rect.Bottom);
                        pt3 = new PointF(rect.X - startwidth, rect.Bottom);
                        path.AddPolygon(new PointF[] { pt1, pt2, pt3, pt4 });
                        path.Reverse();
                        Graphics.FillPath(new SolidBrush(ptrRange.Color), path);
                    }
                }
            }
        }
        /// <summary>
        /// Used to draw the Pointers. 
        /// </summary>
        /// <param name="Graphics">Graphics</param>
        public void DrawPointer(Graphics Graphics)
        {
            Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            GraphicsPath path = new GraphicsPath();
            int a = 0;
            if (this.LinearGauge.MinimumValue < 0)
                a = (int)Math.Ceiling((((Math.Abs(this.LinearGauge.MinimumValue) + LinearGauge.Value) / (float)LinearGauge.MajorDifference) * m_minorTicksPixels));
            else
                a = (int)Math.Ceiling((Math.Abs(LinearGauge.Value - this.LinearGauge.MinimumValue) / (float)LinearGauge.MajorDifference) * m_minorTicksPixels);
            int y = (this.LinearGauge.Height / 2 + LinearGauge.MajorTicksHeight) - LinearGauge.MajorTicksHeight;
            Point pt1, pt2, pt3;
            if (LinearGauge.LinearFrameType == LinearFrameType.Vertical)
            {
                a = 10 + (int)Math.Ceiling((majorTicksDistance * m_minorTicksPixels)) - a;

                if (LinearGauge.PointerPlacement == Placement.Near)
                {
                    pt1 = new Point(this.LinearGauge.Width / 2 - 15, 10 + a);
                    pt2 = new Point(this.LinearGauge.Width / 2 - 15, 20 + a);
                    pt3 = new Point(this.LinearGauge.Width / 2, 15 + a);
                }
                else if (LinearGauge.PointerPlacement == Placement.Center)
                {
                    pt1 = new Point(this.LinearGauge.Width / 2 + 20, 10 + a);
                    pt2 = new Point(this.LinearGauge.Width / 2 + 20, 20 + a);
                    pt3 = new Point(this.LinearGauge.Width / 2 + 5, 15 + a);
                }
                else
                {
                    pt1 = new Point(this.LinearGauge.Width / 2 + 28, 10 + a);
                    pt2 = new Point(this.LinearGauge.Width / 2 + 28, 20 + a);
                    pt3 = new Point(this.LinearGauge.Width / 2 + 13, 15 + a);
                }

            }
            else
            {
                a = a + 10;

                if (LinearGauge.PointerPlacement == Placement.Near)
                {
                    pt1 = new Point(10 + a, y - 15);
                    pt2 = new Point(20 + a, y - 15);
                    pt3 = new Point(15 + a, y);
                }
                else if (LinearGauge.PointerPlacement == Placement.Center)
                {
                    pt1 = new Point(10 + a, y + 20);
                    pt2 = new Point(20 + a, y + 20);
                    pt3 = new Point(15 + a, y + 5);
                }
                else
                {
                    pt1 = new Point(10 + a, y + 27);
                    pt2 = new Point(20 + a, y + 27);
                    pt3 = new Point(15 + a, y + 12);
                }

            }
            path.AddPolygon(new Point[] { pt1, pt2, pt3 });
            Graphics.FillPath((new SolidBrush(LinearGauge.NeedleColor)), path);
        }

        /// <summary>
        /// Updates the Render
        /// </summary>
        /// <param name="PaintEventArgs">PaintEventArgs</param>
        public void UpdateRenderer(PaintEventArgs PaintEventArgs)
        {
            DrawFrame(PaintEventArgs.Graphics);
            DrawLines(PaintEventArgs.Graphics);
            DrawRanges(PaintEventArgs.Graphics);
            DrawPointer(PaintEventArgs.Graphics);
        }

        /// <summary>
        /// To convert number as String
        /// </summary>
        /// <param name="n"></param>
        /// <returns></returns>
        public string NumberToStringRepresentation(int n)
        {
            if (n < 0)
                return "Minus " + NumberToStringRepresentation(-n);
            else if (n == 0)
                return "";
            else if (n <= 19)
                return new string[] {"One", "Two", "Three", "Four", "Five", "Six", "Seven", "Eight", 
         "Nine", "Ten", "Eleven", "Twelve", "Thirteen", "Fourteen", "Fifteen", "Sixteen", 
         "Seventeen", "Eighteen", "Nineteen"}[n - 1] + " ";
            else if (n <= 99)
                return new string[] {"Twenty", "Thirty", "Forty", "Fifty", "Sixty", "Seventy", 
         "Eighty", "Ninety"}[n / 10 - 2] + " " + NumberToStringRepresentation(n % 10);
            else if (n <= 199)
                return "One Hundred " + NumberToStringRepresentation(n % 100);
            else if (n <= 999)
                return NumberToStringRepresentation(n / 100) + "Hundreds " + NumberToStringRepresentation(n % 100);
            else if (n <= 1999)
                return "One Thousand " + NumberToStringRepresentation(n % 1000);
            else if (n <= 999999)
                return NumberToStringRepresentation(n / 1000) + "Thousands " + NumberToStringRepresentation(n % 1000);
            else if (n <= 1999999)
                return "One Million " + NumberToStringRepresentation(n % 1000000);
            else if (n <= 999999999)
                return NumberToStringRepresentation(n / 1000000) + "Millions " + NumberToStringRepresentation(n % 1000000);
            else if (n <= 1999999999)
                return "One Billion " + NumberToStringRepresentation(n % 1000000000);
            else
                return NumberToStringRepresentation(n / 1000000000) + "Billions " + NumberToStringRepresentation(n % 1000000000);
        }

        #endregion
    }
}
